searchData={"items":[{"type":"module","title":"ExTermbox","doc":"Elixir bindings for the termbox2 library, providing a way to control the\nterminal and draw simple UIs using Erlang NIFs.\n\nThis module provides the main user-facing API. It interacts with the termbox2\nNIF functions via the `ExTermbox.Server` GenServer, which must be running.","ref":"ExTermbox.html"},{"type":"module","title":"Architecture - ExTermbox","doc":"`ExTermbox` relies on a `GenServer`, typically registered as `ExTermbox.Server`,\nto manage the lifecycle of the `termbox2` NIF library and handle asynchronous\nevents. Most functions in this module are thin wrappers that send messages\n(calls or casts) to this server process.\n\nSee `ExTermbox.Server` for implementation details.","ref":"ExTermbox.html#module-architecture"},{"type":"module","title":"Usage - ExTermbox","doc":"1.  **Initialization:** Call `ExTermbox.init/1` to start the `ExTermbox.Server`.\n    This calls `tb_init()` via the NIF and registers the calling process\n    as the 'owner' to receive events.\n2.  **API Calls:** Use functions like `change_cell/5`, `clear/0`, `print/5`, etc.\n    These functions communicate with the running `ExTermbox.Server`.\n3.  **Display:** Call `ExTermbox.present/0` to synchronize the internal back\n    buffer with the terminal screen.\n4.  **Events:** The `ExTermbox.Server` automatically polls for terminal events\n    (keyboard, mouse, resize) using `tb_peek_event()` via the NIF. Events are\n    parsed into `%ExTermbox.Event{}` structs and sent as messages in the format\n    `{:termbox_event, event}` to the owner process (the one that called `init/1`).\n    You do not need to manually poll for events.\n5.  **Shutdown:** Call `ExTermbox.shutdown/0` when finished.\n    This stops the `ExTermbox.Server` gracefully, which in turn calls\n    `tb_shutdown()` via the NIF.","ref":"ExTermbox.html#module-usage"},{"type":"module","title":"Event Handling - ExTermbox","doc":"Events are delivered automatically as messages in the format `{:termbox_event, %ExTermbox.Event{}}`\nto the process that called `init/1`. You should handle these messages in your\nprocess's `handle_info/2` callback (if it's a GenServer or similar OTP process).\n\nExample (`handle_info` in the owner process):\n\n```elixir\ndef handle_info({:termbox_event, %ExTermbox.Event{type: :key, key: :q}}, state) do\n  IO.puts(\"Quit key pressed!\")\n  # Initiate shutdown sequence\n  ExTermbox.shutdown()\n  {:stop, :normal, state}\nend\n\ndef handle_info({:termbox_event, event}, state) do\n  IO.inspect(event, label: \"Received Termbox Event\")\n  # Handle other events (resize, mouse, other keys)\n  {:noreply, state}\nend\n```","ref":"ExTermbox.html#module-event-handling"},{"type":"function","title":"ExTermbox.change_cell/6","doc":"Changes the character, foreground, and background attributes of a specific cell\nin the internal back buffer by sending a request to the `ExTermbox.Server`.\n\nThe server calls the `termbox2` NIF function `tb_set_cell()`.\nThis does not immediately affect the visible terminal; `present/1` must be called\nto synchronize.\n\nArguments:\n  - `x`: The zero-based column index.\n  - `y`: The zero-based row index.\n  - `char`: The character to place in the cell. Can be:\n      - An integer codepoint (e.g., `?a`).\n      - A single-character string (e.g., `\"a\"`).\n      - A single-codepoint UTF-8 string (e.g., `\"€\"`).\n  - `fg`: The foreground attribute (an integer constant from `ExTermbox.Constants`).\n    Combine colors (e.g., `Constants.color(:red)`) with attributes\n    (e.g., `Constants.attribute(:bold)`) using bitwise OR (`Bitwise.bor/2`).\n  - `bg`: The background attribute (an integer constant from `ExTermbox.Constants`).\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).\n\nReturns `:ok` if the arguments are valid and the request is sent.\nReturns `{:error, :invalid_char}` if the `char` argument is not a valid single\ncharacter representation.\n\nThis function uses `GenServer.cast`, so errors during the NIF call itself\nwill not be reported back directly but may be logged by the server.","ref":"ExTermbox.html#change_cell/6"},{"type":"function","title":"ExTermbox.clear/1","doc":"Clears the internal back buffer by sending a request to the `ExTermbox.Server`.\n\nThe server calls the `termbox2` NIF function `tb_clear()`.\nThis does not immediately affect the visible terminal; `present/1` must be called\nto synchronize.\n\nArguments:\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).\n\nReturns `:ok` on success or `{:error, reason}` if the GenServer call fails.","ref":"ExTermbox.html#clear/1"},{"type":"function","title":"ExTermbox.debug_crash/1","doc":"[Debug] Causes the C helper process to exit immediately.\nFOR TESTING ONLY.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#debug_crash/1"},{"type":"function","title":"ExTermbox.get_cell/3","doc":"Retrieves the character, foreground, and background attributes of a specific cell\nby querying the `ExTermbox.Server`.\n\n**Note:** This function is not currently implemented in v2.0.0. The underlying\n`tb_get_cell/2` function is not available in the current termbox2_nif version.\nCalls will return `{:error, {:not_implemented, \"...\"}}`.\n\nThis feature may be added in a future version when the NIF library is updated.\n\nReturns `{:ok, {char_codepoint, fg_attribute, bg_attribute}}` on success,\nor `{:error, reason}` on failure.\n\nArguments:\n  - `x`: The zero-based column index.\n  - `y`: The zero-based row index.\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).","ref":"ExTermbox.html#get_cell/3"},{"type":"function","title":"ExTermbox.height/1","doc":"Returns the height of the terminal by querying the `ExTermbox.Server`.\n\nThe server retrieves this information via the `termbox2` NIF function `tb_height()`.\n\nArguments:\n  - `server`: The registered name or PID of the server (defaults to `Elixir.ExTermbox.Server`).","ref":"ExTermbox.html#height/1"},{"type":"function","title":"ExTermbox.init/1","doc":"Initializes the termbox library by starting the `ExTermbox.Server` GenServer.\n\nThis function attempts to start and link an `ExTermbox.Server` process.\nThe server process, upon its own initialization, will call the underlying\n`termbox2` NIF function `tb_init()`.\n\nThe calling process is registered as the \"owner\" of the termbox session and\nwill receive `{:termbox_event, %ExTermbox.Event{}}` messages.\n\nReturns `{:ok, server_name}` on success, where `server_name` is the atom used\nto register the GenServer (defaults to `ExTermbox.Server`). Returns `{:error, reason}`\nif the server cannot be started or if `tb_init()` fails within the server.\n\nIf the server is already running under the specified name, it logs a warning\nand returns `{:ok, server_name}` without attempting to start a new one.\n\nOptions:\n  - `:name` (atom): The registered name for the `ExTermbox.Server` GenServer.\n    Defaults to `ExTermbox.Server`.\n  - `:owner` (pid): The PID to receive termbox events. Defaults to `self()`.\n    This is typically not overridden directly, as `init/1` sets it.\n  - `:poll_interval_ms` (pos_integer): The interval in milliseconds for polling\n    terminal events via `tb_peek_event`. Defaults to `10`.\n\nAll options are passed down to `ExTermbox.Server.start_link/1`.","ref":"ExTermbox.html#init/1"},{"type":"type","title":"ExTermbox.pid_or_name/0","doc":"","ref":"ExTermbox.html#t:pid_or_name/0"},{"type":"function","title":"ExTermbox.present/1","doc":"Synchronizes the internal back buffer with the terminal screen by sending a\nrequest to the `ExTermbox.Server`.\n\nThe server calls the `termbox2` NIF function `tb_present()`.\n\nArguments:\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).\n\nReturns `:ok` on success or `{:error, reason}` if the GenServer call fails.","ref":"ExTermbox.html#present/1"},{"type":"function","title":"ExTermbox.print/6","doc":"A convenience function to print a string at a given position with specified attributes.\n\nThis function iterates through the string's characters and calls `change_cell/6`\nfor each one, sending multiple requests to the `ExTermbox.Server`.\n\nNote: This function assumes a left-to-right character display and does not handle\nline wrapping or terminal boundaries explicitly. Characters printed beyond the\nterminal width might be ignored by the underlying `termbox2` library.\n\nArguments:\n  - `x`: The starting zero-based column index.\n  - `y`: The zero-based row index.\n  - `fg`: The foreground attribute (integer constant from `ExTermbox.Constants`).\n  - `bg`: The background attribute (integer constant from `ExTermbox.Constants`).\n  - `str`: The string to print.\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).\n\nReturns `:ok`. Like `change_cell/6`, this uses `GenServer.cast`, so errors during\nindividual NIF calls are not reported directly.","ref":"ExTermbox.html#print/6"},{"type":"function","title":"ExTermbox.select_input_mode/2","doc":"Selects the input mode by sending a request to the `ExTermbox.Server`.\n\nThe server validates the mode atom against `ExTermbox.Constants.input_mode/1`\nand then calls the `termbox2` NIF function `tb_set_input_mode()`.\n\nArguments:\n  - `mode`: An input mode atom defined in `ExTermbox.Constants` (e.g., `:esc`, `:alt`, `:mouse`).\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).\n\nReturns `:ok` on success, `{:error, :invalid_input_mode}` if the mode atom is\nunrecognized, or `{:error, reason}` for other GenServer call errors.","ref":"ExTermbox.html#select_input_mode/2"},{"type":"function","title":"ExTermbox.set_clear_attributes/3","doc":"Sets the clear attributes (foreground and background) used by `clear/1`\nby sending a request to the `ExTermbox.Server`.\n\nThe server validates the attributes and calls the `termbox2` NIF function\n`tb_set_clear_attrs()`.\n\nArguments:\n  - `fg`: The foreground attribute (an integer constant from `ExTermbox.Constants`).\n  - `bg`: The background attribute (an integer constant from `ExTermbox.Constants`).\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).\n\nReturns `:ok` on success, or `{:error, reason}` if the GenServer call fails.","ref":"ExTermbox.html#set_clear_attributes/3"},{"type":"function","title":"ExTermbox.set_cursor/3","doc":"Sets the cursor position by sending a request to the `ExTermbox.Server`.\n\nThe server calls the `termbox2` NIF function `tb_set_cursor()`.\n\nUse `x = -1` and `y = -1` (or the default arguments) to hide the cursor.\nSee `ExTermbox.Constants.hide_cursor/0`.\n\nArguments:\n  - `x`: The zero-based column index (-1 to hide).\n  - `y`: The zero-based row index (-1 to hide).\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).\n\nReturns `:ok`. This function uses `GenServer.cast`, so errors during the NIF call\nwill not be reported back directly but may be logged by the server.","ref":"ExTermbox.html#set_cursor/3"},{"type":"function","title":"ExTermbox.set_output_mode/2","doc":"Sets the output mode by sending a request to the `ExTermbox.Server`.\n\nThe server validates the mode atom against `ExTermbox.Constants.output_mode/1`\nand then calls the `termbox2` NIF function `tb_set_output_mode()`.\n\nArguments:\n  - `mode`: An output mode atom defined in `ExTermbox.Constants` (e.g., `:normal`, `:grayscale`, `:xterm256`).\n  - `server`: The registered name or PID of the server (defaults to `#{inspect(@server_name)}`).\n\nReturns `:ok` on success, `{:error, :invalid_output_mode}` if the mode atom is\nunrecognized, or `{:error, reason}` for other GenServer call errors.","ref":"ExTermbox.html#set_output_mode/2"},{"type":"function","title":"ExTermbox.shutdown/1","doc":"Shuts down the termbox library by stopping the `ExTermbox.Server` GenServer.\n\nThis function finds the `ExTermbox.Server` process (using the provided or\ndefault name) and requests it to stop gracefully (using `:shutdown`).\n\nThe server's `terminate/2` callback is responsible for calling the `termbox2`\nNIF function `tb_shutdown()` to restore the terminal state.\n\nReturns `:ok` if the stop request was sent or if the server was not running.\nReturns `{:error, reason}` if finding the server process fails unexpectedly.\n\nArguments:\n  - `server_name`: The registered name of the server to stop.\n    Defaults to `ExTermbox.Server`.","ref":"ExTermbox.html#shutdown/1"},{"type":"function","title":"ExTermbox.start_link/1","doc":"","ref":"ExTermbox.html#start_link/1"},{"type":"function","title":"ExTermbox.width/1","doc":"Returns the width of the terminal by querying the `ExTermbox.Server`.\n\nThe server retrieves this information via the `termbox2` NIF function `tb_width()`.\n\nArguments:\n  - `server`: The registered name or PID of the server (defaults to `Elixir.ExTermbox.Server`).","ref":"ExTermbox.html#width/1"},{"type":"module","title":"ExTermbox.Cell","doc":"Represents a termbox cell, a character at a position, along with the cell's\nbackground and foreground colors.","ref":"ExTermbox.Cell.html"},{"type":"function","title":"ExTermbox.Cell.empty/0","doc":"","ref":"ExTermbox.Cell.html#empty/0"},{"type":"type","title":"ExTermbox.Cell.t/0","doc":"","ref":"ExTermbox.Cell.html#t:t/0"},{"type":"module","title":"ExTermbox.Constants","doc":"Defines constants from the termbox2 library. These can be used e.g. to set\nformatting attributes, colors, input/output modes, or to identify keys passed\nin an event.\n\nValues are based on termbox2 v2.5.0 (or compatible).","ref":"ExTermbox.Constants.html"},{"type":"type","title":"ExTermbox.Constants.attribute/0","doc":"","ref":"ExTermbox.Constants.html#t:attribute/0"},{"type":"function","title":"ExTermbox.Constants.attribute/1","doc":"Retrieves an attribute constant by name","ref":"ExTermbox.Constants.html#attribute/1"},{"type":"function","title":"Examples - ExTermbox.Constants.attribute/1","doc":"iex> attribute(:bold)\n    0x0100\n    iex> attribute(:underline)\n    0x0200\n    iex> attribute(:italic)\n    0x0800","ref":"ExTermbox.Constants.html#attribute/1-examples"},{"type":"function","title":"ExTermbox.Constants.attributes/0","doc":"Retrieves the mapping of attribute constants (assuming 16-bit attributes).","ref":"ExTermbox.Constants.html#attributes/0"},{"type":"type","title":"ExTermbox.Constants.color/0","doc":"","ref":"ExTermbox.Constants.html#t:color/0"},{"type":"function","title":"ExTermbox.Constants.color/1","doc":"Retrieves a color constant by name","ref":"ExTermbox.Constants.html#color/1"},{"type":"function","title":"Examples - ExTermbox.Constants.color/1","doc":"iex> color(:red)\n    0x02\n    iex> color(:blue)\n    0x05","ref":"ExTermbox.Constants.html#color/1-examples"},{"type":"function","title":"ExTermbox.Constants.colors/0","doc":"Retrieves the mapping of color constants (basic 8 colors + default).","ref":"ExTermbox.Constants.html#colors/0"},{"type":"type","title":"ExTermbox.Constants.constant/0","doc":"","ref":"ExTermbox.Constants.html#t:constant/0"},{"type":"type","title":"ExTermbox.Constants.error_code/0","doc":"","ref":"ExTermbox.Constants.html#t:error_code/0"},{"type":"function","title":"ExTermbox.Constants.error_code/1","doc":"Retrieves an error code constant by name","ref":"ExTermbox.Constants.html#error_code/1"},{"type":"function","title":"Examples - ExTermbox.Constants.error_code/1","doc":"iex> error_code(:unsupported_term)\n    -17\n    iex> error_code(:mem)\n    -5\n    iex> error_code(:ok)\n    0","ref":"ExTermbox.Constants.html#error_code/1-examples"},{"type":"function","title":"ExTermbox.Constants.error_codes/0","doc":"Retrieves the mapping of error code constants.","ref":"ExTermbox.Constants.html#error_codes/0"},{"type":"type","title":"ExTermbox.Constants.event_type/0","doc":"","ref":"ExTermbox.Constants.html#t:event_type/0"},{"type":"function","title":"ExTermbox.Constants.event_type/1","doc":"Retrieves an event type constant by name","ref":"ExTermbox.Constants.html#event_type/1"},{"type":"function","title":"Examples - ExTermbox.Constants.event_type/1","doc":"iex> event_type(:key)\n    0x01\n    iex> event_type(:resize)\n    0x02\n    iex> event_type(:mouse)\n    0x03","ref":"ExTermbox.Constants.html#event_type/1-examples"},{"type":"function","title":"ExTermbox.Constants.event_types/0","doc":"Retrieves the mapping of event type constants.","ref":"ExTermbox.Constants.html#event_types/0"},{"type":"function","title":"ExTermbox.Constants.hide_cursor/0","doc":"Retrieves the hide cursor constant.","ref":"ExTermbox.Constants.html#hide_cursor/0"},{"type":"function","title":"Examples - ExTermbox.Constants.hide_cursor/0","doc":"iex> hide_cursor()\n    -1","ref":"ExTermbox.Constants.html#hide_cursor/0-examples"},{"type":"type","title":"ExTermbox.Constants.hide_cursor/0","doc":"","ref":"ExTermbox.Constants.html#t:hide_cursor/0"},{"type":"type","title":"ExTermbox.Constants.input_mode/0","doc":"","ref":"ExTermbox.Constants.html#t:input_mode/0"},{"type":"function","title":"ExTermbox.Constants.input_mode/1","doc":"Retrieves an input mode constant by name","ref":"ExTermbox.Constants.html#input_mode/1"},{"type":"function","title":"Examples - ExTermbox.Constants.input_mode/1","doc":"iex> input_mode(:esc)\n    1\n    iex> input_mode(:mouse)\n    4\n    iex> input_mode(:alt_with_mouse)\n    6","ref":"ExTermbox.Constants.html#input_mode/1-examples"},{"type":"function","title":"ExTermbox.Constants.input_modes/0","doc":"Retrieves the mapping of input mode constants.","ref":"ExTermbox.Constants.html#input_modes/0"},{"type":"type","title":"ExTermbox.Constants.key/0","doc":"","ref":"ExTermbox.Constants.html#t:key/0"},{"type":"function","title":"ExTermbox.Constants.key/1","doc":"Retrieves a key constant by name","ref":"ExTermbox.Constants.html#key/1"},{"type":"function","title":"Examples - ExTermbox.Constants.key/1","doc":"iex> key(:esc)\n    0x1B\n    iex> key(:space)\n    0x20\n    iex> key(:back_tab)\n    65513","ref":"ExTermbox.Constants.html#key/1-examples"},{"type":"function","title":"ExTermbox.Constants.keys/0","doc":"Retrieves the mapping of key constants for use with termbox2.\n\nThese are based on terminfo constants. Note that there's some overlap\nof terminfo values. For example, it's not possible to distinguish between\n` ` and `ctrl-h`.","ref":"ExTermbox.Constants.html#keys/0"},{"type":"function","title":"ExTermbox.Constants.mod/1","doc":"Retrieves a modifier constant by name","ref":"ExTermbox.Constants.html#mod/1"},{"type":"function","title":"Examples - ExTermbox.Constants.mod/1","doc":"iex> mod(:alt)\n    1\n    iex> mod(:none)\n    0\n    iex> mod(:ctrl)\n    2\n    iex> mod(:motion)\n    8","ref":"ExTermbox.Constants.html#mod/1-examples"},{"type":"type","title":"ExTermbox.Constants.modifier/0","doc":"","ref":"ExTermbox.Constants.html#t:modifier/0"},{"type":"function","title":"ExTermbox.Constants.modifiers/0","doc":"Retrieves the mapping of modifier constants.","ref":"ExTermbox.Constants.html#modifiers/0"},{"type":"type","title":"ExTermbox.Constants.output_mode/0","doc":"","ref":"ExTermbox.Constants.html#t:output_mode/0"},{"type":"function","title":"ExTermbox.Constants.output_mode/1","doc":"Retrieves an output mode constant by name","ref":"ExTermbox.Constants.html#output_mode/1"},{"type":"function","title":"Examples - ExTermbox.Constants.output_mode/1","doc":"iex> output_mode(:normal)\n    1\n    iex> output_mode(:term_256)\n    2\n    iex> output_mode(:truecolor)\n    5","ref":"ExTermbox.Constants.html#output_mode/1-examples"},{"type":"function","title":"ExTermbox.Constants.output_modes/0","doc":"Retrieves the mapping of output mode constants.","ref":"ExTermbox.Constants.html#output_modes/0"},{"type":"function","title":"ExTermbox.Constants.resolve_color/1","doc":"Resolves a color atom (e.g., :red) or integer to its integer value.\nReturns the default color value if the input is invalid or not found.\n\nThis primarily works for the basic 8 colors defined in `@colors`.\nFor extended color modes (256, truecolor, etc.), pass the integer directly.\nAttributes can also be passed as integers and will be returned as-is.","ref":"ExTermbox.Constants.html#resolve_color/1"},{"type":"module","title":"ExTermbox.Event","doc":"Represents an event received from the termbox library.\n\nEvents are polled by `ExTermbox.Server` and sent to the owner process.","ref":"ExTermbox.Event.html"},{"type":"type","title":"ExTermbox.Event.t/0","doc":"The event structure.\n\nFields:\n* `:type` - The type of event (e.g., `:key`, `:resize`, `:mouse`). Atom.\n* `:mod` - Modifier keys pressed (e.g., `:alt`). Atom or nil.\n* `:key` - The key pressed (e.g., `:f1`, `:arrow_up`, `:ctrl_a`). Atom or nil.\n* `:ch` - The character pressed (if applicable, Unicode codepoint). Integer or nil.\n* `:w` - New width (for resize events). Integer or nil.\n* `:h` - New height (for resize events). Integer or nil.\n* `:x` - Mouse x position (for mouse events). Integer or nil.\n* `:y` - Mouse y position (for mouse events). Integer or nil.","ref":"ExTermbox.Event.html#t:t/0"},{"type":"module","title":"ExTermbox.Position","doc":"Represents a position on the screen by encoding a pair of cartesian\ncoordinates. The origin is the top-left-most character on the screen\n`(0, 0)`, while x and y increase from left to right and top to bottom,\nrespectively.","ref":"ExTermbox.Position.html"},{"type":"type","title":"ExTermbox.Position.t/0","doc":"","ref":"ExTermbox.Position.html#t:t/0"},{"type":"function","title":"ExTermbox.Position.translate/3","doc":"Translates (shifts) a position by some delta x and y.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate/3"},{"type":"function","title":"Examples - ExTermbox.Position.translate/3","doc":"iex> translate(%Position{x: 0, y: 0}, 1, 2)\n    %Position{x: 1, y: 2}\n    iex> translate(%Position{x: 10, y: 0}, -1, 0)\n    %Position{x: 9, y: 0}","ref":"ExTermbox.Position.html#translate/3-examples"},{"type":"function","title":"ExTermbox.Position.translate_x/2","doc":"Translates a position by a delta x.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate_x/2"},{"type":"function","title":"Examples - ExTermbox.Position.translate_x/2","doc":"iex> translate_x(%Position{x: 0, y: 0}, 2)\n    %Position{x: 2, y: 0}\n    iex> translate_x(%Position{x: 2, y: 0}, -1)\n    %Position{x: 1, y: 0}","ref":"ExTermbox.Position.html#translate_x/2-examples"},{"type":"function","title":"ExTermbox.Position.translate_y/2","doc":"Translates a position by a delta y.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate_y/2"},{"type":"function","title":"Examples - ExTermbox.Position.translate_y/2","doc":"iex> translate_y(%Position{x: 0, y: 0}, 2)\n    %Position{x: 0, y: 2}\n    iex> translate_y(%Position{x: 0, y: 2}, -1)\n    %Position{x: 0, y: 1}","ref":"ExTermbox.Position.html#translate_y/2-examples"},{"type":"module","title":"ExTermbox.Server","doc":"GenServer managing the Termbox NIF lifecycle and event polling.","ref":"ExTermbox.Server.html"},{"type":"function","title":"ExTermbox.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExTermbox.Server.html#child_spec/1"},{"type":"function","title":"ExTermbox.Server.start_link/1","doc":"","ref":"ExTermbox.Server.html#start_link/1"},{"type":"extras","title":"Raxol Revived ExTermbox","doc":"# Raxol Revived ExTermbox\n\n[![Hex.pm](https://img.shields.io/hexpm/v/rrex_termbox.svg)](https://hex.pm/packages/rrex_termbox)\n[![Hexdocs.pm](https://img.shields.io/badge/api-hexdocs-brightgreen.svg)](https://hexdocs.pm/rrex_termbox)\n\nAn Elixir library for interacting with the terminal via the [termbox2](https://github.com/termbox/termbox2) C library, a maintained fork of the original termbox.\n\n**Starting with version 2.0.0, this library uses Elixir Native Implemented Functions (NIFs) provided by the [`termbox2`](https://hex.pm/packages/termbox2) Hex package (and its associated NIF bindings).** This replaces the previous Port/Unix Domain Socket architecture, leveraging a maintained C library and simplifying the build process.\n\nFor high-level, declarative terminal UIs in Elixir, see [raxol](https://github.com/Hydepwns/raxol) or its predecessor [Ratatouille](https://github.com/ndreynolds/ratatouille), which build on top of this library.\n\nFor the API Reference, see the `ExTermbox` module: [https://hexdocs.pm/rrex_termbox/ExTermbox.html](https://hexdocs.pm/rrex_termbox/ExTermbox.html).","ref":"readme.html"},{"type":"extras","title":"Getting Started - Raxol Revived ExTermbox","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Architecture (NIF Based) - Raxol Revived ExTermbox","doc":"**Note:** If you previously used versions prior to 2.0.0, be aware that the underlying communication mechanism has changed significantly from a Port/UDS system back to NIFs, leveraging the `termbox2` dependency. See the [Changelog](./CHANGELOG.md) for details.\n\n`ExTermbox` now interacts directly with the `termbox2` C library through NIFs provided by the `termbox2` Hex dependency.\n\n1. **Initialization:** `ExTermbox.init/1` starts a `GenServer` (`ExTermbox.Server`) which calls the `tb_init()` NIF function. This server manages the termbox state and handles API calls.\n2. **API Calls:** Public functions in the `ExTermbox` module (e.g., `ExTermbox.print/5`, `ExTermbox.clear/0`, `ExTermbox.present/0`) communicate with the `ExTermbox.Server` via `GenServer` calls/casts. The server then invokes the corresponding `termbox2` NIF function (e.g., `tb_print`, `tb_clear`, `tb_present`).\n3. **Event Handling:** The `ExTermbox.Server` periodically polls for terminal events (like key presses, mouse events, or resizes) using the `tb_peek_event()` NIF. When an event occurs, it's translated into an `ExTermbox.Event` struct and sent as a standard Elixir message (`{:termbox_event, event}`) to the process that originally called `ExTermbox.init/1` (the \"owner\" process).\n\nThe public API is exposed primarily through the `ExTermbox` module.","ref":"readme.html#architecture-nif-based"},{"type":"extras","title":"Hello World - Raxol Revived ExTermbox","doc":"Let's go through a simple example.\n\nCreate an Elixir script (e.g., `hello.exs`) in any Mix project that includes `rrex_termbox` in its dependencies (see Installation below).\n\n```elixir\n# hello.exs\ndefmodule HelloWorld do\n  use GenServer\n  alias ExTermbox\n\n  def start_link(_opts) do\n    # Start our process that will own the termbox session\n    GenServer.start_link(__MODULE__, :ok, name: __MODULE__)\n  end\n\n  @impl true\n  def init(:ok) do\n    # Initialize ExTermbox, registering this GenServer process as the owner\n    # The owner process will receive {:termbox_event, event} messages\n    case ExTermbox.init(self()) do\n      :ok ->\n        IO.puts(\"ExTermbox initialized successfully.\")\n        # Send ourselves a message to trigger drawing the initial screen\n        send(self(), :draw)\n        {:ok, %{}} # Initial state for our GenServer\n\n      {:error, reason} ->\n        IO.inspect(reason, label: \"Error initializing ExTermbox\")\n        {:stop, :init_failed}\n    end\n  end\n\n  @impl true\n  def handle_info(:draw, state) do\n    # Clear the screen\n    :ok = ExTermbox.clear()\n\n    # Print \"Hello, World!\" at (0, 0) with default colors\n    :ok = ExTermbox.print(0, 0, :default, :default, \"Hello, World!\")\n\n    # Print \"(Press   to quit)\" at (0, 2)\n    :ok = ExTermbox.print(0, 2, :default, :default, \"(Press   to quit)\")\n\n    # Render the changes to the terminal\n    :ok = ExTermbox.present()\n    {:noreply, state}\n  end\n\n  # Handle events sent from ExTermbox.Server\n  @impl true\n  def handle_info({:termbox_event, %ExTermbox.Event{type: :key, key: :q}}, state) do\n    IO.puts(\"Quit event received.\")\n    # Trigger shutdown before stopping\n    ExTermbox.shutdown()\n    {:stop, :normal, state}\n  end\n\n  @impl true\n  def handle_info({:termbox_event, event}, state) do\n    # Optional: Log other events\n    # IO.inspect(event, label: \"Received event\")\n    {:noreply, state}\n  end\n\n  # Handle other messages if needed\n  @impl true\n  def handle_info(msg, state) do\n    # IO.inspect(msg, label: \"Received other message\")\n    {:noreply, state}\n  end\n\n  @impl true\n  def terminate(reason, _state) do\n    IO.puts(\"HelloWorld GenServer terminating: #{inspect(reason)}\")\n    # Ensure termbox is shut down if termination wasn't triggered by :q\n    # (This might be redundant if ExTermbox.Server links/monitors)\n    ExTermbox.shutdown()\n    :ok\n  end\n\n  # Helper to run the example\n  def run do\n    # Ensure the app is started if running as a script\n    {:ok, _} = Application.ensure_all_started(:rrex_termbox)\n\n    {:ok, pid} = start_link([])\n    # Keep the script alive until the GenServer terminates\n    Process.monitor(pid)\n    receive do\n      {:DOWN, _, :process, ^pid, reason} ->\n        IO.puts(\"HelloWorld process finished: #{inspect(reason)}\")\n    end\n  end\nend\n\nHelloWorld.run()\n```\n\nIn this example, we use a `GenServer` to manage the application's lifecycle and handle the asynchronous `{:termbox_event, ...}` messages.\n\nFinally, run the example like this (assuming you have `rrex_termbox` added to a Mix project):\n\n```bash\nmix run hello.exs\n```\n\nYou should see the text we rendered and be able to quit with 'q'.","ref":"readme.html#hello-world"},{"type":"extras","title":"Installation - Raxol Revived ExTermbox","doc":"Add `rrex_termbox` as a dependency in your project's `mix.exs`.\n\n**Important:** This library currently relies on a fork of the `termbox2` NIF wrapper to include necessary fixes and features. Point your dependency directly to the GitHub repository:\n\n```elixir\ndef deps do\n  [\n    # {:rrex_termbox, \"~> 2.0.0\"}, # Use this once published to Hex\n    {:rrex_termbox, git: \"https://github.com/Hydepwns/rrex_termbox.git\", tag: \"v2.0.0-alpha.2\"} # Or branch: \"main\"\n\n    # The underlying NIF library (currently points to a fork)\n    # rrex_termbox depends on this, so it's usually fetched automatically,\n    # but explicitly listing it might be needed for overrides.\n    # {:termbox2, github: \"Hydepwns/termbox2-nif\", tag: \"0.1.1-hydepwns-fix1\", submodules: true}\n  ]\nend\n```\n\n*(Note: Once `rrex_termbox` v2.0.0 (or later) is published on Hex.pm and the underlying `termbox2` dependency issues are resolved upstream or the fork is published, the dependency specification can likely be simplified back to `{:rrex_termbox, \"~> 2.0.0\"}`.)*\n\nYou will need standard C build tools (like `gcc` or `clang`, often part of `build-essential` or Xcode Command Line Tools) installed on your system for the `termbox2` NIF dependency to compile.\n\nMix should handle fetching the dependency and compiling the NIFs automatically when you run `mix deps.get` and `mix compile`.\n\nIf you encounter build issues, ensure your build tools are installed and check the `termbox2` dependency's documentation or repository for any specific requirements.","ref":"readme.html#installation"},{"type":"extras","title":"Distribution - Raxol Revived ExTermbox","doc":"Building standalone releases for applications using `rrex_termbox` (and its underlying NIF dependency) should work with standard Elixir **[Releases](https://hexdocs.pm/mix/Mix.Tasks.Release.html)**. The build process compiles the NIFs into a shared object file (`.so` or `.dylib`) located in the `priv/` directory of the dependency (`termbox2`). Releases are designed to package these `priv/` artifacts correctly.\n\nEnsure your release configuration properly includes the `rrex_termbox` and `termbox2` applications. Consult the Elixir Releases documentation for details.","ref":"readme.html#distribution"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}